#define __cacheflush RENAMED___cacheflush
#define cacheflush RENAMED_cacheflush
#include <errno.h>
#include <shlib-compat.h>
#undef __cacheflush
#undef cacheflush
long int _no_syscall (void)
{ __set_errno (ENOSYS); return -1L; }
weak_alias (_no_syscall, __cacheflush)
stub_warning (__cacheflush)
weak_alias (_no_syscall, __GI___cacheflush)
weak_alias (_no_syscall, cacheflush)
stub_warning (cacheflush)
weak_alias (_no_syscall, __GI_cacheflush)
